/*------------------------------------------------*/
/* znakoveSady/utf8fce.c                          */
#include <stdio.h>
#include "utf8fce.h"

int utf8bytesInChar(const char text) {
    if(!text) {
        return 0;
    }
    else if((text & (char) 128) == 0) {
        return 1;
    }
    else if((unsigned char)(text & 224) == 192) {
        return 2;
    }
    else if((unsigned char)(text & 240) == 224) {
        return 3;
    }
    else if((unsigned char)(text & 248) == 240) {
        return 4;
    }
    return 0;
}

int utf8moveToNextChar(const char *text) {
    int i = 0;
    int bytes;
    bytes = utf8bytesInChar(*text);
    if(!bytes) return -1;
    for(;bytes >= 1; bytes--) {
        if(!text[i]) { return -2; }
        i++;
    }
    return i;
}

int utf8strlen(const char *text) {
    size_t i = 0, len = 0;
    int move;

    while(text[i]) {
        len++;
        move = utf8moveToNextChar(text+i);
        if(move < 0) { 
            fprintf(stderr, "Chybný UTF-8 znak %u!\n", i);
            return -1;
        }
        i += move;
    }
    return len;
}

char * utf8CharAtIndex(const char *text, const size_t index, size_t *next) {
    static char znak[MAX_BYTES_IN_UTF8+1];
    size_t i = 0, len = 0, move;
    int bytes;

    while(text[i]) {
        if(index == len) {
            bytes = utf8bytesInChar(*(text+i));
            for(len = 0; len < bytes; len++) {
                znak[len] = *(text+i+len);
            }
            znak[len] = '\0';
            if(next != NULL) {
                 *next = i+len;
            }
            return znak;
        }
        len++;
        move = utf8moveToNextChar(text+i);
        if(move < 0) { 
            fprintf(stderr, "Chybny UTF-8 znak %u!\n", i);
            return NULL;
        }
        i += move;
    }
    return NULL;
}
/*------------------------------------------------*/
